#include <stdio.h>
#include <string.h>
#include <stddef.h>
#include <sys/time.h>
#include <stdlib.h>

#include <e-hal.h>

typedef unsigned int BF_word;
typedef BF_word BF_binary[6];

typedef struct
{
	BF_binary BF_out[16] __attribute__ ((packed));
	int test[16] __attribute__ ((packed));
	int start[16] __attribute__ ((packed));
	int core_done[16] __attribute__ ((packed));
	char key[16][72+1] __attribute__ ((packed));
	char hash[16][61] __attribute__ ((packed));
	int done __attribute__ ((packed));
} data;	

#define BufOffset 0x01000000
#define BufSize sizeof(data)

#define ERR(x,s) \
if((x) == E_ERR) {\
	fprintf(stderr, s); \
	exit(1);\
}

struct fmt_tests {
	char *ciphertext, *plaintext;
	char *fields[10];
};

static struct fmt_tests test[] = {
	{"$2a$05$CCCCCCCCCCCCCCCCCCCCC.E5YPO9kmyuRGyh0XouQYb4YMJKvyOeW",
		"U*U"},
	{"$2a$05$CCCCCCCCCCCCCCCCCCCCC.VGOzA784oUp/Z0DY336zx7pLYAy0lwK",
		"U*U*"},
	{"$2a$05$XXXXXXXXXXXXXXXXXXXXXOAcXxm9kjPGEMsLznoKqmqw7tc8WCx4a",
		"U*U*U"},
	{"$2a$05$CCCCCCCCCCCCCCCCCCCCC.7uG0VCzI2bS7j6ymqJi9CdcdxiRTWNy",
		""},
	{"$2a$05$abcdefghijklmnopqrstuu5s2v8.iXieOjg/.AySBTTZIIVFJeBui",
		"0123456789abcdefghijklmnopqrstuvwxyz"
		"ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"
		"chars after 72 are ignored"},
	{"$2x$05$/OK.fbVrR/bpIqNJ5ianF.CE5elHaaO4EbggVDjb8P19RukzXSM3e",
		"\xa3"},
	{"$2y$05$/OK.fbVrR/bpIqNJ5ianF.Sa7shbm4.OzKpvFnX1pQLmQW96oUlCq",
		"\xa3"},
	{"$2x$05$6bNw2HLQYeqHYyBfLMsv/OiwqTymGIGzFsA4hOTWebfehXHNprcAS",
		"\xd1\x91"},
	{"$2x$05$6bNw2HLQYeqHYyBfLMsv/O9LIGgn8OMzuDoHfof8AQimSGfcSWxnS",
		"\xd0\xc1\xd2\xcf\xcc\xd8"},
	{"$2a$05$/OK.fbVrR/bpIqNJ5ianF.swQOIzjOiJ9GHEPuhEkvqrUyvWhEMx6",
		"\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa"
		"\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa"
		"\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa"
		"\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa"
		"\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa"
		"\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa"
		"chars after 72 are ignored as usual"},
	{"$2a$05$/OK.fbVrR/bpIqNJ5ianF.R9xrDjiycxMbQE2bp.vgqlYpW5wx2yy",
		"\xaa\x55\xaa\x55\xaa\x55\xaa\x55\xaa\x55\xaa\x55"
		"\xaa\x55\xaa\x55\xaa\x55\xaa\x55\xaa\x55\xaa\x55"
		"\xaa\x55\xaa\x55\xaa\x55\xaa\x55\xaa\x55\xaa\x55"
		"\xaa\x55\xaa\x55\xaa\x55\xaa\x55\xaa\x55\xaa\x55"
		"\xaa\x55\xaa\x55\xaa\x55\xaa\x55\xaa\x55\xaa\x55"
		"\xaa\x55\xaa\x55\xaa\x55\xaa\x55\xaa\x55\xaa\x55"},
	{"$2a$05$CCCCCCCCCCCCCCCCCCCCC.7uG0VCzI2bS7j6ymqJi9CdcdxiRTWNy",
		""},
	{"$2a$05$/OK.fbVrR/bpIqNJ5ianF.9tQZzcJfm3uj2NvJ/n5xkhpqLrMpWCe",
		"\x55\xaa\xff\x55\xaa\xff\x55\xaa\xff\x55\xaa\xff"
		"\x55\xaa\xff\x55\xaa\xff\x55\xaa\xff\x55\xaa\xff"
		"\x55\xaa\xff\x55\xaa\xff\x55\xaa\xff\x55\xaa\xff"
		"\x55\xaa\xff\x55\xaa\xff\x55\xaa\xff\x55\xaa\xff"
		"\x55\xaa\xff\x55\xaa\xff\x55\xaa\xff\x55\xaa\xff"
		"\x55\xaa\xff\x55\xaa\xff\x55\xaa\xff\x55\xaa\xff"},
	{NULL}
};


int main(int argc, char *argv[])
{
	e_platform_t platform;
	e_epiphany_t dev;
	e_mem_t emem;
	data result;
	struct timeval start, end;
	int i = 0;
	
	ERR(e_init(NULL), "Initialization of platform failed!\n");	
	
	ERR(e_reset_system(), "Reset of Epiphany chip failed!\n");
	
	ERR(e_get_platform_info(&platform), "Get platform info failed!\n");
	
	ERR(e_alloc(&emem, BufOffset, BufSize), "Epiphany memory allocation failed!\n");

	ERR(e_open(&dev, 0, 0, platform.rows, platform.cols), "e_open() failed!\n");
	
	ERR(e_load_group("e_bcrypt.srec", &dev, 0, 0, platform.rows, platform.cols, e_true), "Load failed!\n");
	
	for(i = 0; i < platform.rows*platform.cols; i++)
		result.start[i] = 16;
		
	ERR(e_write(&emem, 0, 0, offsetof(data, start), result.start, sizeof(result.start)), "Writing start failed!\n");
	
	gettimeofday(&start, NULL);	
	
	//while(result.core_done[0] == 0)
	sleep(1);
		e_read(&emem, 0, 0, 0x0, &result, BufSize);
		
	gettimeofday(&end, NULL);
	
	fprintf(stderr, "Execution time - Epiphany: %f ms\n", (double)(end.tv_sec - start.tv_sec)*1000 + ((double)(end.tv_usec - start.tv_usec)/1000.0));
	fprintf(stderr, "done = %d\n", result.done);
		
	for(i = 0; i < 16; i++)
		fprintf(stderr, "core_done[%2d] = %d\t test[%2d] = %d\t start[%2d] = %d\n", i, result.core_done[i], i, result.test[i], i, result.start[i]);
			
	ERR(e_close(&dev), "Closing Epiphany chip failed!\n");
	ERR(e_free(&emem), "Freeing memory failed!\n");
	ERR(e_finalize(), "e_finalize failed!\n");

	return 0;
}
